#!/bin/bash
#script to automate the pseudo-multithreaded version of Gate

(  
    #open the system monitor (& allows the terminal to continue running)
       gnome-system-monitor &

    #delete all previous outputs and remake the output folders
    (
        rm -rfv ./output/*
        mkdir ./output/core1
        mkdir ./output/core2
        mkdir ./output/core3
        mkdir ./output/core4
        mkdir ./output/core5
        mkdir ./output/core6
        mkdir ./output/core7
        mkdir ./output/core8
        mkdir ./output/merged
    )

    (
    #get parameters for passing through to Gate
    #start with setting number of primaries
        NumberPrimaries=0
        read -p "How many primaries per core?" NumberPrimaries    
        echo "You have chosen $NumberPrimaries primaries to run on each core"

    #ask for CT data file using Dialog (a GUI for picking a file)
        printf "Please select the CT data folder %b\nTab to move between windows and use spacebar to copy the current folder. %b\nExit using the Enter Key when the folder path is in the textbox"

        CTDataFile=$(dialog --title "Select CT Data" --fselect ./data/dcm/CT/ 10 10 --stdout)

        #get the dialog's exit status for checking if it was cancelled later        
        return_value=$?

        #remove first two characters to match the mac file (i.e. remove ./)
        CTDataFile="${CTDataFile:2}"

        #act on exit status (esac escapes case)
        case $return_value in
        0)
            echo "You have chosen $CTDataFile";;
        1)
            echo "CT Data choice cancelled";;
        esac

    #ask for the PT data file using Dialog (a GUI for a picking a file)
        printf "Please select the PET data folder %b\nTab to move between windows and use spacebar to copy the current folder. %b\nExit using the Enter Key when the folder path is in the textbox"

        PTDataFile=$(dialog --title "Select PET Data" --fselect ./data/dcm/PT/ 10 10 --stdout)
    
        #get the dialogs exit status for checking if it was cancelled
        return_value=$?

        #remove first two characters to match the mac file (i.e. remove ./)
        PTDataFile="${PTDataFile:2}"

        #act on exit status (esac escapes case)
        case $return_value in
        0)
            echo "You have chosen $PTDataFile";;
        1)
            echo "PT Data choice cancelled";;
        esac

    )


    #run several instances of gate to utilise all cores
    (
    echo "Starting Gate..."
        Gate mac/main1.mac [CTData, $CTDataFile] [NoOfPrimaries,$NumberPrimaries] [PETData,$PTDataFile] &
        #Gate mac/main2.mac -a [CTData, $CTDataFile] [NoOfPrimaries,$NumberPrimaries] [PETData,$PETDataFile] &
        #Gate mac/main3.mac -a [CTData, $CTDataFile] [NoOfPrimaries,$NumberPrimaries] [PETData,$PETDataFile] &
        #Gate mac/main4.mac -a [CTData, $CTDataFile] [NoOfPrimaries,$NumberPrimaries] [PETData,$PETDataFile] &
        #Gate mac/main5.mac -a [CTData, $CTDataFile] [NoOfPrimaries,$NumberPrimaries] [PETData,$PETDataFile] &
        #Gate mac/main6.mac -a [CTData, $CTDataFile] [NoOfPrimaries,$NumberPrimaries] [PETData,$PETDataFile] &
        #Gate mac/main7.mac -a [CTData, $CTDataFile] [NoOfPrimaries,$NumberPrimaries] [PETData,$PETDataFile] &
        #Gate mac/main8.mac -a [CTData, $CTDataFile] [NoOfPrimaries,$NumberPrimaries] [PETData,$PETDataFile] &
        wait
    )

    #copy the root files to the main directory, merge them, move the merged file and delete the copied files
    (
        cp ./output/core1/trackLength1.root ./
        cp ./output/core2/trackLength2.root ./
        cp ./output/core3/trackLength3.root ./
        cp ./output/core4/trackLength4.root ./
        cp ./output/core1/trackLength5.root ./
        cp ./output/core2/trackLength6.root ./
        cp ./output/core3/trackLength7.root ./
        cp ./output/core4/trackLength8.root ./
        hadd trackLengthMerged.root trackLength1.root trackLength2.root trackLength3.root trackLength4.root trackLength5.root trackLength6.root trackLength7.root trackLength8.root

        mv ./trackLengthMerged.root ./output/merged/
        rm ./trackLength1.root ./trackLength2.root ./trackLength3.root ./trackLength4.root ./trackLength5.root ./trackLength6.root ./trackLength7.root ./trackLength8.root
    )

    (
        cp ./output/core1/energySpectrum1.root ./
        cp ./output/core2/energySpectrum2.root ./
        cp ./output/core3/energySpectrum3.root ./
        cp ./output/core4/energySpectrum4.root ./
        cp ./output/core1/energySpectrum5.root ./
        cp ./output/core2/energySpectrum6.root ./
        cp ./output/core3/energySpectrum7.root ./
        cp ./output/core4/energySpectrum8.root ./
        hadd energySpectrumMerged.root energySpectrum1.root energySpectrum2.root energySpectrum3.root energySpectrum4.root energySpectrum5.root energySpectrum6.root energySpectrum7.root energySpectrum8.root

        mv ./energySpectrumMerged.root ./output/merged/
        rm ./energySpectrum1.root ./energySpectrum2.root ./energySpectrum3.root ./energySpectrum4.root ./energySpectrum5.root ./energySpectrum6.root ./energySpectrum7.root ./energySpectrum8.root
    )

    #copy mhd files to single location then merge them with ImageJ
    (
        #Look for a file with a name containing the extension in any subfolder of /output/       
        #Copy the file from the result (i.e. {}) to the place I want, 
        #the \; lets bash know to move on to the next file 
        find ./output/* -iname '*.mhd' -exec cp {} './' \;
        find ./output/* -iname '*.raw' -exec cp {} './' \;    

        #run imageJ in headless mode, pass filenames of MHDs and link to macro to produce the merged MHDs
        #separate imageJ macro written which is pointed towards, 
        #just need to pass a variable for the filename after "PatientDoseX-"
        cd ~/Softwares/Fiji.app
        ./ImageJ-linux64 --ij2 --headless -macro ./plugins/Macros/mergeMHD.ijm 'Dose'
        wait

        ./ImageJ-linux64 --ij2 --headless -macro ./plugins/Macros/mergeMHD.ijm 'Dose-Squared'
        wait

        ./ImageJ-linux64 --ij2 --headless -macro ./plugins/Macros/mergeMHD.ijm 'Dose-Uncertainty'
        wait

        ./ImageJ-linux64 --ij2 --headless -macro ./plugins/Macros/mergeMHD.ijm 'Nbofhits'
        wait

        ./ImageJ-linux64 --ij2 --headless -macro ./plugins/Macros/mergeMHD.ijm 'Edep-Squared'
        wait

        ./ImageJ-linux64 --ij2 --headless -macro ./plugins/Macros/mergeMHD.ijm 'Edep-Uncertainty'
        wait

        ./ImageJ-linux64 --ij2 --headless -macro ./plugins/Macros/mergeMHD.ijm 'Edep'
        wait
        cd ~/gate-exercises/trialimportsplit

        #just cleans up all of the temporary files from the main folder
        rm ./*.mhd ./*.raw ./*.mha
    )

    #copy sundry files like txts and stats into merged location
    #used similar code as for MHDs above, just looking for txt only
    (
        find ./output/* -iname '*.txt' -exec cp {} './output/merged/' \;
    )

)
